@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# This file is part of the Wapiti project (https://wapiti.sourceforge.io)
-# Copyright (C) 2008-2021 Nicolas Surribas
+# This file is part of the Wapiti project (https://wapiti-scanner.github.io)
+# Copyright (C) 2008-2022 Nicolas Surribas
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 import os
-import sys
 from os.path import splitext, join as path_join
 from urllib.parse import quote, urlparse
 from collections import defaultdict
@@ -27,43 +26,47 @@ import random
 from types import GeneratorType, FunctionType
 from binascii import hexlify
 from functools import partialmethod
+from pkg_resources import resource_filename
 
 from httpx import ReadTimeout, RequestError
 
 from wapitiCore.language.vulnerability import CRITICAL_LEVEL, HIGH_LEVEL, MEDIUM_LEVEL, LOW_LEVEL, INFO_LEVEL
 from wapitiCore.net.web import Request
 
-# All Wapiti attack modules
-modules = [
-    "mod_crlf",
-    "mod_csp",
-    "mod_http_headers",
-    "mod_csrf",
-    "mod_cookieflags",
-    "mod_drupal_enum",
-    "mod_exec",
-    "mod_file",
-    "mod_sql",
-    "mod_xss",
-    "mod_backup",
-    "mod_brute_login_form",
-    "mod_htaccess",
-    "mod_timesql",
-    "mod_permanentxss",
-    "mod_nikto",
-    "mod_buster",
-    "mod_shellshock",
-    "mod_methods",
-    "mod_ssrf",
-    "mod_redirect",
-    "mod_xxe",
-    "mod_wapp",
-    "mod_wp_enum",
-    "mod_takeover"
-]
+
+all_modules = {
+    "backup",
+    "brute_login_form",
+    "buster",
+    "cookieflags",
+    "crlf",
+    "csp",
+    "csrf",
+    "drupal_enum",
+    "exec",
+    "file",
+    "htaccess",
+    "htp",
+    "http_headers",
+    "log4shell",
+    "methods",
+    "nikto",
+    "permanentxss",
+    "redirect",
+    "shellshock",
+    "sql",
+    "ssl",
+    "ssrf",
+    "takeover",
+    "timesql",
+    "wapp",
+    "wp_enum",
+    "xss",
+    "xxe"
+}
 
 # Modules that will be used if option -m isn't used
-commons = [
+common_modules = {
     "cookieflags",
     "csp",
     "exec",
@@ -72,9 +75,25 @@ commons = [
     "permanentxss",
     "redirect",
     "sql",
+    "ssl",
     "ssrf",
     "xss"
-]
+}
+
+# Modules that will be used in passive mode -m passive
+passive_modules = {
+    "cookieflags",
+    "csp",
+    "http_headers",
+    "wapp"
+}
+
+presets = {
+    "all": all_modules,
+    "common": common_modules,
+    "passive": passive_modules
+}
+
 
 VULN = "vulnerability"
 ANOM = "anomaly"
@@ -121,12 +140,12 @@ def random_string_with_flags():
 
 class Flags:
     def __init__(
-        self,
-        payload_type=PayloadType.pattern,
-        section="",
-        method=PayloadType.get,
-        platform="all",
-        dbms="all"
+            self,
+            payload_type=PayloadType.pattern,
+            section="",
+            method=PayloadType.get,
+            platform="all",
+            dbms="all"
     ):
         self.payload_type = payload_type
         self.section = section
@@ -166,11 +185,11 @@ class Flags:
             raise ValueError("Can't compare a Flags object to another kind of object")
 
         return (
-            self.payload_type == other.payload_type and
-            self.section == other.section and
-            self.method == other.method and
-            self.platform == other.platform and
-            self.dbms == other.dbms
+                self.payload_type == other.payload_type and
+                self.section == other.section and
+                self.method == other.method and
+                self.platform == other.platform and
+                self.dbms == other.dbms
         )
 
 
@@ -186,9 +205,7 @@ class Attack:
     # Must be defined in the code of the module
     require = []
 
-    BASE_DIR = os.path.dirname(sys.modules["wapitiCore"].__file__)
-    DATA_DIR = os.path.join(BASE_DIR, "data", "attacks")
-
+    DATA_DIR = resource_filename("wapitiCore", os.path.join("data", "attacks"))
     HOME_DIR = os.getenv("HOME") or os.getenv("USERPROFILE")
 
     PAYLOADS_FILE = None
@@ -214,6 +231,10 @@ class Attack:
     # The priority of the module, from 0 (first) to 10 (last). Default is 5
     PRIORITY = 5
 
+    @staticmethod
+    def get_resource(resource_path: str):
+        return resource_filename("wapitiCore", path_join("data", "attacks", resource_path))
+
     def __init__(self, crawler, persister, attack_options, stop_event):
         super().__init__()
         self._session_id = "".join([random.choice("0123456789abcdefghjijklmnopqrstuvwxyz") for __ in range(0, 6)])
@@ -235,7 +256,7 @@ class Attack:
         self.deps = []
 
     async def add_payload(self, payload_type: str, category: str, request_id: int = -1,
-                          level=0, request=None, parameter="", info=""):
+                          level=0, request=None, parameter="", info="", wstg=None):
         await self.persister.add_payload(
             request_id=request_id,
             payload_type=payload_type,
@@ -244,9 +265,11 @@ class Attack:
             level=level,
             request=request,
             parameter=parameter,
-            info=info
+            info=info,
+            wstg=wstg
         )
 
+    add_vuln = partialmethod(add_payload, payload_type=VULN)
     add_vuln_critical = partialmethod(add_payload, payload_type=VULN, level=CRITICAL_LEVEL)
     add_vuln_high = partialmethod(add_payload, payload_type=VULN, level=HIGH_LEVEL)
     add_vuln_medium = partialmethod(add_payload, payload_type=VULN, level=MEDIUM_LEVEL)
@@ -256,7 +279,7 @@ class Attack:
     add_anom_high = partialmethod(add_payload, payload_type=ANOM, level=HIGH_LEVEL)
     add_anom_medium = partialmethod(add_payload, payload_type=ANOM, level=MEDIUM_LEVEL)
 
-    add_addition = partialmethod(add_payload, payload_type=ADDITION, level=LOW_LEVEL)
+    add_addition = partialmethod(add_payload, payload_type=ADDITION, level=INFO_LEVEL)
 
     @property
     def payloads(self):
@@ -272,6 +295,10 @@ class Attack:
     def attack_level(self):
         return self.options.get("level", 1)
 
+    @property
+    def dns_endpoint(self):
+        return self.options.get("dns_endpoint", "dns.wapiti3.ovh")
+
     @property
     def internal_endpoint(self):
         return self.options.get("internal_endpoint", "https://wapiti3.ovh/")
@@ -602,70 +629,3 @@ class PayloadReader:
         clean_line = clean_line.replace("\\0", "\0")
 
         return clean_line, Flags(payload_type=flag_type)
-
-
-if __name__ == "__main__":
-
-    mutator = Mutator(payloads=[("INJECT", Flags()), ("ATTACK", Flags())], qs_inject=True, max_queries_per_pattern=16)
-    res1 = Request(
-        "http://httpbin.org/post?var1=a&var2=b",
-        post_params=[['post1', 'c'], ['post2', 'd']]
-    )
-
-    res2 = Request(
-        "http://httpbin.org/post?var1=a&var2=z",
-        post_params=[['post1', 'c'], ['post2', 'd']]
-    )
-
-    res3 = Request(
-        "http://httpbin.org/get?login=admin&password=letmein",
-    )
-
-    assert res1.hash_params == res2.hash_params
-
-    for evil_request, param_name, payload, flags in mutator.mutate(res1):
-        print(evil_request)
-        print(flags)
-
-    print('')
-    print("#" * 50)
-    print('')
-
-    for evil_request, param_name, payload, flags in mutator.mutate(res2):
-        print(evil_request)
-
-    print('')
-    print("#" * 50)
-    print('')
-
-    def iterator():
-        yield "abc", Flags()
-        yield "def", Flags()
-
-    mutator = Mutator(payloads=iterator, qs_inject=True, max_queries_per_pattern=16)
-    for evil_request, param_name, payload, flags in mutator.mutate(res3):
-        print(evil_request)
-
-    print('')
-    print("#" * 50)
-    print('')
-
-    mutator = Mutator(payloads=random_string_with_flags, qs_inject=True, max_queries_per_pattern=16)
-    for evil_request, param_name, payload, flags in mutator.mutate(res3):
-        print(evil_request)
-        print("Payload is", payload)
-
-    mutator = Mutator(
-        methods="G",
-        payloads=[("INJECT", Flags()), ("ATTACK", Flags())],
-        qs_inject=True,
-        parameters=["var1"]
-    )
-
-    assert len(list(mutator.mutate(res1))) == 2
-
-    f1 = Flags()
-    f2 = Flags()
-    assert f1 == f2
-    assert f1.with_section("abcd") == f2.with_section("abcd")
-    assert f1 != f1.with_section("abcd")
