@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# This file is part of the Wapiti project (https://wapiti.sourceforge.io)
-# Copyright (C) 2006-2021 Nicolas SURRIBAS
+# This file is part of the Wapiti project (https://wapiti-scanner.github.io)
+# Copyright (C) 2006-2022 Nicolas SURRIBAS
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,45 +16,42 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-import sys
 import argparse
+import asyncio
+import codecs
 import os
-from urllib.parse import urlparse
-from time import strftime, gmtime
-from importlib import import_module
-from operator import attrgetter
-from traceback import print_tb
+import signal
+import sys
 from collections import deque
 from datetime import datetime
-from uuid import uuid1
-from sqlite3 import OperationalError
 from hashlib import sha256
-from random import choice
-import codecs
+from importlib import import_module
 from inspect import getdoc
-import asyncio
-import signal
-from typing import AsyncGenerator
+from operator import attrgetter
+from random import choice
+from sqlite3 import OperationalError
+from time import gmtime, strftime
+from traceback import print_tb
+from typing import AsyncGenerator, Dict, List
+from urllib.parse import urlparse
+from uuid import uuid1
 
 import browser_cookie3
 import httpx
 from httpx import RequestError
-
+from wapitiCore.attack.attack import (Attack, all_modules, common_modules,
+                                      presets)
+from wapitiCore.definitions import (additionals, anomalies, flatten_references,
+                                    vulnerabilities)
 from wapitiCore.language.language import _
-from wapitiCore.definitions import anomalies, additionals, vulnerabilities, flatten_references
-
-from wapitiCore.net import crawler, jsoncookie
-from wapitiCore.net.web import Request
-from wapitiCore.report import get_report_generator_instance, GENERATORS
-from wapitiCore.net.sql_persister import SqlPersister
-from wapitiCore.moon import phase
 from wapitiCore.main.log import logging
+from wapitiCore.moon import phase
+from wapitiCore.net import crawler, jsoncookie
+from wapitiCore.net.sql_persister import SqlPersister
+from wapitiCore.net.web import Request
+from wapitiCore.report import GENERATORS, get_report_generator_instance
 
-from wapitiCore.attack import attack
-
-BASE_DIR = None
-WAPITI_VERSION = "Wapiti 3.0.8"
-CONF_DIR = os.path.dirname(sys.modules["wapitiCore"].__file__)
+WAPITI_VERSION = "Wapiti 3.1.0"
 
 SCAN_FORCE_VALUES = {
     "paranoid": 1,
@@ -96,6 +93,75 @@ def module_to_class_name(module_name: str) -> str:
     return "Module" + module_name.title().replace("_", "")
 
 
+def activate_method_module(module: Attack, method: str, status: bool):
+    if not method:
+        module.do_get = module.do_post = status
+    elif method == "get":
+        module.do_get = status
+    elif method == "post":
+        module.do_post = status
+
+
+def filter_modules_with_options(module_options: str, loaded_modules: Dict[str, Attack]) -> List[Attack]:
+    activated_modules: Dict[str, Attack] = {}
+
+    if module_options == "":
+        return []
+
+    if module_options is None:
+        # Default is to use common modules
+        module_options = "common"
+
+    for module_opt in module_options.split(","):
+        if module_opt.strip() == "":
+            # Trailing comma, etc
+            continue
+
+        method = ""
+        if module_opt.find(":") > 0:
+            module_name, method = module_opt.split(":", 1)
+        else:
+            module_name = module_opt
+
+        if module_name.startswith("-"):
+            # The whole module or some of the methods needs to be deactivated
+            module_name = module_name[1:]
+
+            for bad_module in presets.get(module_name, [module_name]):
+                if bad_module not in loaded_modules:
+                    logging.error(_("[!] Unable to find a module named {0}").format(bad_module))
+                    continue
+
+                if bad_module not in activated_modules:
+                    # You can't deactivate a module that is not used
+                    continue
+
+                if not method:
+                    activated_modules.pop(bad_module)
+                else:
+                    activate_method_module(activated_modules[bad_module], method, False)
+        else:
+            # The whole module or some of the methods needs to be deactivated
+            if module_name.startswith("+"):
+                module_name = module_name[1:]
+
+            for good_module in presets.get(module_name, [module_name]):
+                if good_module not in loaded_modules:
+                    logging.error(_("[!] Unable to find a module named {0}").format(good_module))
+                    continue
+
+                if good_module in activated_modules:
+                    continue
+
+                if good_module not in activated_modules:
+                    activated_modules[good_module] = loaded_modules[good_module]
+
+                if method:
+                    activate_method_module(activated_modules[good_module], method, False)
+
+    return sorted(activated_modules.values(), key=attrgetter("PRIORITY"))
+
+
 class Wapiti:
     """This class parse the options from the command line and set the modules and the HTTP engine accordingly.
     Launch wapiti without arguments or with the "-h" option for more information."""
@@ -200,7 +266,7 @@ class Wapiti:
     def history_file(self):
         return self._history_file
 
-    def _init_report(self):
+    async def _init_report(self):
         self.report_gen = get_report_generator_instance(self.report_generator_type.lower())
 
         self.report_gen.set_report_info(
@@ -208,7 +274,8 @@ class Wapiti:
             self.target_scope,
             gmtime(),
             WAPITI_VERSION,
-            self._auth_state
+            self._auth_state,
+            await self.count_resources()
         )
 
         for vul in vulnerabilities:
@@ -216,7 +283,8 @@ class Wapiti:
                 vul.NAME,
                 vul.DESCRIPTION,
                 vul.SOLUTION,
-                flatten_references(vul.REFERENCES)
+                flatten_references(vul.REFERENCES),
+                vul.WSTG_CODE
             )
 
         for anomaly in anomalies:
@@ -224,7 +292,8 @@ class Wapiti:
                 anomaly.NAME,
                 anomaly.DESCRIPTION,
                 anomaly.SOLUTION,
-                flatten_references(anomaly.REFERENCES)
+                flatten_references(anomaly.REFERENCES),
+                anomaly.WSTG_CODE
             )
 
         for additional in additionals:
@@ -232,119 +301,59 @@ class Wapiti:
                 additional.NAME,
                 additional.DESCRIPTION,
                 additional.SOLUTION,
-                flatten_references(additional.REFERENCES)
+                flatten_references(additional.REFERENCES),
+                additional.WSTG_CODE
             )
 
-    def _init_attacks(self, stop_event: asyncio.Event):
-        self._init_report()
+    async def _init_attacks(self, stop_event: asyncio.Event):
+        await self._init_report()
         stop_event.clear()
 
-        logging.info(_("[*] Loading modules:"))
-        modules_list = sorted(module_name[4:] for module_name in attack.modules)
-        logging.info(f"\t {', '.join(modules_list)}")
-        for mod_name in attack.modules:
+        logging.info(_("[*] Existing modules:"))
+        logging.info(f"\t {', '.join(sorted(all_modules))}")
+
+        modules = {}
+        for mod_name in all_modules:
             try:
-                mod = import_module("wapitiCore.attack." + mod_name)
-            except ImportError:
-                logging.error(_("[!] Could not find module {0}").format(mod_name))
-                continue
-
-            class_name = module_to_class_name(mod_name)
-            class_instance = getattr(mod, class_name)(self.crawler, self.persister, self.attack_options, stop_event)
-            if hasattr(class_instance, "set_timeout"):
-                class_instance.set_timeout(self.crawler.timeout)
-            self.attacks.append(class_instance)
-
-            self.attacks.sort(key=attrgetter("PRIORITY"))
-
-        for attack_module in self.attacks:
-            if attack_module.name not in attack.commons:
-                attack_module.do_get = False
-                attack_module.do_post = False
-
-        # Custom list of modules was specified
-        if self.module_options is not None:
-            # First deactivate all modules
-            for attack_module in self.attacks:
-                attack_module.do_get = False
-                attack_module.do_post = False
-
-            opts = self.module_options.split(",")
-
-            for module_opt in opts:
-                if module_opt.strip() == "":
+                try:
+                    mod = import_module("wapitiCore.attack.mod_" + mod_name)
+                except ImportError:
+                    logging.error(_("[!] Unable to import module {0}").format(mod_name))
                     continue
 
-                method = ""
-                if module_opt.find(":") > 0:
-                    module_name, method = module_opt.split(":", 1)
-                else:
-                    module_name = module_opt
+                class_name = module_to_class_name(mod_name)
+                class_instance = getattr(mod, class_name)(self.crawler, self.persister, self.attack_options, stop_event)
+                if hasattr(class_instance, "set_timeout"):
+                    class_instance.set_timeout(self.crawler.timeout)
 
-                # deactivate some module options
-                if module_name.startswith("-"):
-                    module_name = module_name[1:]
-                    if module_name in ("all", "common"):
-                        for attack_module in self.attacks:
-                            if module_name == "all" or attack_module.name in attack.commons:
-                                if not method:
-                                    attack_module.do_get = attack_module.do_post = False
-                                elif method == "get":
-                                    attack_module.do_get = False
-                                elif method == "post":
-                                    attack_module.do_post = False
-                    else:
-                        found = False
-                        for attack_module in self.attacks:
-                            if attack_module.name == module_name:
-                                found = True
-                                if not method:
-                                    attack_module.do_get = attack_module.do_post = False
-                                elif method == "get":
-                                    attack_module.do_get = False
-                                elif method == "post":
-                                    attack_module.do_post = False
-                        if not found:
-                            logging.error(_("[!] Unable to find a module named {0}").format(module_name))
+            except Exception as exception:
+                # Catch every possible exceptions and print it
+                logging.error(_("[!] Module {0} seems broken and will be skipped").format(mod_name))
+                logging.exception(exception.__class__.__name__, exception)
+                continue
 
-                # activate some module options
-                else:
-                    if module_name.startswith("+"):
-                        module_name = module_name[1:]
+            modules[mod_name] = class_instance
 
-                    if module_name in ("all", "common"):
-                        for attack_module in self.attacks:
-                            if module_name == "all" or attack_module.name in attack.commons:
-                                if not method:
-                                    attack_module.do_get = attack_module.do_post = True
-                                elif method == "get":
-                                    attack_module.do_get = True
-                                elif method == "post":
-                                    attack_module.do_post = True
-                    else:
-                        found = False
-                        for attack_module in self.attacks:
-                            if attack_module.name == module_name:
-                                found = True
-                                if not method:
-                                    attack_module.do_get = attack_module.do_post = True
-                                elif method == "get":
-                                    attack_module.do_get = True
-                                elif method == "post":
-                                    attack_module.do_post = True
-                        if not found:
-                            logging.error(_("[!] Unable to find a module named {0}").format(module_name))
+        self.attacks = filter_modules_with_options(self.module_options, modules)
 
     async def update(self):
         """Update modules that implement an update method"""
         stop_event = asyncio.Event()
-        for mod_name in attack.modules:
-            mod = import_module("wapitiCore.attack." + mod_name)
-            class_name = module_to_class_name(mod_name)
-            class_instance = getattr(mod, class_name)(self.crawler, self.persister, self.attack_options, stop_event)
-            if hasattr(class_instance, "update"):
-                logging.info(_("Updating module {0}").format(mod_name[4:]))
-                await class_instance.update()
+        for mod_name in all_modules:
+            try:
+                mod = import_module("wapitiCore.attack.mod_" + mod_name)
+                class_name = module_to_class_name(mod_name)
+                class_instance = getattr(mod, class_name)(self.crawler, self.persister, self.attack_options, stop_event)
+                if hasattr(class_instance, "update"):
+                    logging.info(_("Updating module {0}").format(mod_name))
+                    await class_instance.update()
+            except ImportError:
+                continue
+            except Exception:  # pylint: disable=broad-except
+                # Catch every possible exceptions and print it
+                logging.error(_("[!] Module {0} seems broken and will be skipped").format(mod_name))
+                continue
+
         logging.success(_("Update done."))
 
     async def load_scan_state(self):
@@ -397,7 +406,7 @@ class Wapiti:
         # Let's save explorer values (limits)
         explorer.save_state(self.persister.output_file[:-2] + "pkl")
 
-    async def load_resources_for_module(self, module: attack.Attack) -> AsyncGenerator[Request, None]:
+    async def load_resources_for_module(self, module: Attack) -> AsyncGenerator[Request, None]:
         if module.do_get:
             async for resource in self.persister.get_links(attack_module=module.name):
                 yield resource
@@ -407,7 +416,7 @@ class Wapiti:
 
     async def attack(self, stop_event: asyncio.Event):
         """Launch the attacks based on the preferences set by the command line"""
-        self._init_attacks(stop_event)
+        await self._init_attacks(stop_event)
         answer = "0"
 
         for attack_module in self.attacks:
@@ -561,6 +570,7 @@ class Wapiti:
                     parameter=payload.parameter,
                     info=payload.info,
                     module=payload.module,
+                    wstg=payload.wstg
                 )
             elif payload.type == "anomaly":
                 self.report_gen.add_anomaly(
@@ -570,6 +580,7 @@ class Wapiti:
                     parameter=payload.parameter,
                     info=payload.info,
                     module=payload.module,
+                    wstg=payload.wstg
                 )
             elif payload.type == "additional":
                 self.report_gen.add_additional(
@@ -579,6 +590,7 @@ class Wapiti:
                     parameter=payload.parameter,
                     info=payload.info,
                     module=payload.module,
+                    wstg=payload.wstg
                 )
 
         print('')
@@ -1041,6 +1053,34 @@ async def wapiti_main():
         type=int, default=32
     )
 
+    parser.add_argument(
+        "--external-endpoint",
+        metavar="EXTERNAL_ENDPOINT_URL",
+        default=argparse.SUPPRESS,
+        help=_("Url serving as endpoint for target")
+    )
+
+    parser.add_argument(
+        "--internal-endpoint",
+        metavar="INTERNAL_ENDPOINT_URL",
+        default=argparse.SUPPRESS,
+        help=_("Url serving as endpoint for attacker")
+    )
+
+    parser.add_argument(
+        "--endpoint",
+        metavar="ENDPOINT_URL",
+        default="https://wapiti3.ovh/",
+        help=_("Url serving as endpoint for both attacker and target")
+    )
+
+    parser.add_argument(
+        "--dns-endpoint",
+        metavar="DNS_ENDPOINT_DOMAIN",
+        default="dns.wapiti3.ovh",
+        help=_("Domain serving as DNS endpoint for Log4Shell attack")
+    )
+
     parser.add_argument(
         "-t", "--timeout",
         type=float, default=6.0,
@@ -1090,6 +1130,13 @@ async def wapiti_main():
         choices=range(0, 3)
     )
 
+    parser.add_argument(
+        "--log",
+        metavar="OUTPUT_PATH",
+        default=None,
+        help=_("Output log file")
+    )
+
     parser.add_argument(
         "-f", "--format",
         metavar="FORMAT",
@@ -1105,34 +1152,6 @@ async def wapiti_main():
         help=_("Output file or folder")
     )
 
-    parser.add_argument(
-        "--log",
-        metavar="OUTPUT_PATH",
-        default=None,
-        help=_("Output log file")
-    )
-
-    parser.add_argument(
-        "--external-endpoint",
-        metavar="EXTERNAL_ENDPOINT_URL",
-        default=argparse.SUPPRESS,
-        help=_("Url serving as endpoint for target")
-    )
-
-    parser.add_argument(
-        "--internal-endpoint",
-        metavar="INTERNAL_ENDPOINT_URL",
-        default=argparse.SUPPRESS,
-        help=_("Url serving as endpoint for attacker")
-    )
-
-    parser.add_argument(
-        "--endpoint",
-        metavar="ENDPOINT_URL",
-        default="https://wapiti3.ovh/",
-        help=_("Url serving as endpoint for both attacker and target")
-    )
-
     parser.add_argument(
         "--no-bugreport",
         action="store_true",
@@ -1157,14 +1176,16 @@ async def wapiti_main():
 
     if args.list_modules:
         print(_("[*] Available modules:"))
-        modules_list = sorted(module_name[4:] for module_name in attack.modules)
-        for module_name in modules_list:
-            mod = import_module("wapitiCore.attack.mod_" + module_name)
-            class_name = module_to_class_name(module_name)
-            is_common = " (used by default)" if module_name in attack.commons else ""
-            print(f"\t{module_name}{is_common}")
-            print("\t\t" + getdoc(getattr(mod, class_name)))
-            print('')
+        for module_name in sorted(all_modules):
+            try:
+                mod = import_module("wapitiCore.attack.mod_" + module_name)
+                class_name = module_to_class_name(module_name)
+                is_common = " (used by default)" if module_name in common_modules else ""
+                print(f"\t{module_name}{is_common}")
+                print("\t\t" + getdoc(getattr(mod, class_name)))
+                print('')
+            except ImportError:
+                continue
         sys.exit()
 
     url = fix_url_path(args.base_url)
@@ -1214,7 +1235,7 @@ async def wapiti_main():
             wap.set_proxy(args.proxy)
 
         if args.tor:
-            wap.set_proxy("socks://127.0.0.1:9050/")
+            wap.set_proxy("socks5://127.0.0.1:9050/")
 
         if "cookie" in args:
             if os.path.isfile(args.cookie):
@@ -1286,6 +1307,9 @@ async def wapiti_main():
             "tasks": args.tasks
         }
 
+        if "dns_endpoint" in args:
+            attack_options["dns_endpoint"] = args.dns_endpoint
+
         if "endpoint" in args:
             endpoint = fix_url_path(args.endpoint)
             if is_valid_endpoint('ENDPOINT', endpoint):
@@ -1341,9 +1365,13 @@ async def wapiti_main():
                     logging.info(_("[*] Resuming scan from previous session, please wait"))
 
                 if "auth_type" in args:
-                    is_logged_in, form = await wap.crawler.async_try_login(wap.crawler.auth_url, args.auth_type)
+                    is_logged_in, form, excluded_urls = await wap.crawler.async_try_login(
+                        wap.crawler.auth_url,
+                        args.auth_type
+                    )
                     wap.set_auth_state(is_logged_in, form, wap.crawler.auth_url, args.auth_type)
-
+                    for url in excluded_urls:
+                        wap.add_excluded_url(url)
                 await wap.load_scan_state()
                 loop.add_signal_handler(signal.SIGINT, inner_ctrl_c_signal_handler)
                 await wap.browse(global_stop_event, parallelism=args.tasks)
