@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# This file is part of the Wapiti project (https://wapiti.sourceforge.io)
-# Copyright (C) 2006-2021 Nicolas SURRIBAS
+# This file is part of the Wapiti project (https://wapiti-scanner.github.io)
+# Copyright (C) 2006-2022 Nicolas SURRIBAS
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@ import asyncio
 
 # Third-parties
 import httpx
-from httpx_socks import AsyncProxyTransport
 from tld import get_fld
 from tld.exceptions import TldDomainNotFound
 
@@ -74,6 +73,8 @@ EXCLUDED_MEDIA_EXTENSIONS = (
 
 BAD_URL_REGEX = re.compile(r"https?:/[^/]+")
 
+DISCONNECT_REGEX = r'(?i)((log|sign)\s?(out|off)|disconnect|déconnexion)'
+
 
 def wildcard_translate(pattern):
     """Translate a wildcard PATTERN to a regular expression object.
@@ -185,14 +186,16 @@ class AsyncCrawler:
         url_parts = urlparse(proxy)
         protocol = url_parts.scheme.lower()
 
-        if protocol not in ("http", "https", "socks"):
+        if protocol not in ("http", "https", "socks", "socks5"):
             raise ValueError(f"Unknown proxy type: {protocol}")
 
         if protocol == "socks":
-            self._transport = AsyncProxyTransport.from_url(urlunparse(("socks5", url_parts.netloc, '/', '', '', '')))
-        else:
-            proxy_url = urlunparse((url_parts.scheme, url_parts.netloc, '/', '', '', ''))
-            self._proxies = {"http://": proxy_url, "https://": proxy_url}
+            protocol = "socks5"
+
+        self._proxies = {
+            "http://": urlunparse((protocol, url_parts.netloc, '/', '', '', '')),
+            "https://": urlunparse((protocol, url_parts.netloc, '/', '', '', '')),
+        }
 
     @property
     def client(self):
@@ -355,11 +358,14 @@ class AsyncCrawler:
 
             self.client.auth = self._auth
 
-    async def async_try_login(self, auth_url: str, auth_type: str) -> Tuple[bool, dict]:
-        """Try to authenticate with the provided url and credentials."""
+    async def async_try_login(self, auth_url: str, auth_type: str) -> Tuple[bool, dict, List[str]]:
+        """
+        Try to authenticate with the provided url and credentials.
+        Returns if the the authentication has been successful, the used form variables and the disconnect urls.
+        """
         if len(self._auth_credentials) != 2:
             logging.error(_("Login failed") + " : " + _("Invalid credentials format"))
-            return False, {}
+            return False, {}, []
 
         username, password = self._auth_credentials
 
@@ -367,18 +373,33 @@ class AsyncCrawler:
             return await self._async_try_login_post(username, password, auth_url)
         return await self._async_try_login_basic_digest_ntlm(auth_url)
 
-    async def _async_try_login_basic_digest_ntlm(self, auth_url: str) -> Tuple[bool, dict]:
+    async def _async_try_login_basic_digest_ntlm(self, auth_url: str) -> Tuple[bool, dict, List[str]]:
         page = await self.async_get(web.Request(auth_url))
 
-        if page.status in (401, 404):
-            return False, {}
-        return True, {}
+        if page.status in (401, 403, 404):
+            return False, {}, []
+        return True, {}, []
 
-    async def _async_try_login_post(self, username: str, password: str, auth_url: str) -> Tuple[bool, dict]:
+    def _extract_disconnect_urls(self, page: Page) -> List[str]:
+        """
+        Extract all the disconnect urls on the given page and returns them.
+        """
+        disconnect_urls = []
+        for link in page.links:
+            if self.is_in_scope(link) is False:
+                continue
+            element = re.search(DISCONNECT_REGEX, link)
+
+            if element is not None:
+                disconnect_urls.append(page.make_absolute(link))
+        return disconnect_urls
+
+    async def _async_try_login_post(self, username: str, password: str, auth_url: str) -> Tuple[bool, dict, List[str]]:
         # Fetch the login page and try to extract the login form
         try:
             page = await self.async_get(web.Request(auth_url), follow_redirects=True)
             form = {}
+            disconnect_urls = []
 
             login_form, username_field_idx, password_field_idx = page.find_login_form()
             if login_form:
@@ -411,15 +432,17 @@ class AsyncCrawler:
                 )
 
                 # ensure logged in
-                if login_response.soup.find_all(text=re.compile(r'(?i)((log|sign)\s?out|disconnect|déconnexion)')):
+                if login_response.soup.find_all(
+                    text=re.compile(DISCONNECT_REGEX)
+                ):
                     self.is_logged_in = True
                     logging.success(_("Login success"))
-
+                    disconnect_urls = self._extract_disconnect_urls(login_response)
                 else:
                     logging.warning(_("Login failed") + " : " + _("Credentials might be invalid"))
             else:
                 logging.warning(_("Login failed") + " : " + _("No login form detected"))
-            return self.is_logged_in, form
+            return self.is_logged_in, form, disconnect_urls
 
         except ConnectionError:
             logging.error(_("[!] Connection error with URL"), auth_url)
