@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# This file is part of the Wapiti project (https://wapiti.sourceforge.io)
-# Copyright (C) 2006-2021 Nicolas SURRIBAS
+# This file is part of the Wapiti project (https://wapiti-scanner.github.io)
+# Copyright (C) 2006-2022 Nicolas SURRIBAS
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,25 +19,24 @@
 
 # Standard libraries
 import re
-from functools import lru_cache
-from urllib.parse import urlparse, urlunparse
-from hashlib import md5
-from http.client import IncompleteRead
 import warnings
 from ast import literal_eval
+from functools import lru_cache
+from hashlib import md5
+from http.client import IncompleteRead
 from posixpath import normpath
+from urllib.parse import urlparse, urlunparse
+from tld import get_fld
+from tld.exceptions import TldBadUrl, TldDomainNotFound
 
 # Third-parties
 import httpx
-from tld import get_fld
-from tld.exceptions import TldDomainNotFound, TldBadUrl
 from bs4 import BeautifulSoup
 from bs4.element import Comment, Doctype
 
 # Internal libraries
 from wapitiCore import parser_name
-from wapitiCore.net import web
-from wapitiCore.net import lamejs
+from wapitiCore.net import lamejs, web
 
 warnings.filterwarnings(action='ignore', category=UserWarning, module='bs4')
 RE_JS_REDIR = re.compile(
@@ -216,7 +215,7 @@ class Page:
 
         try:
             return literal_eval(self.content)
-        except ValueError:
+        except (ValueError, SyntaxError):
             pass
 
         return None
@@ -701,10 +700,16 @@ class Page:
         urls = set()
         for meta_tag in self.soup.find_all("meta", attrs={"content": True, "http-equiv": True}):
             if meta_tag and meta_tag["http-equiv"].lower() == "refresh":
-                content_str = meta_tag["content"].lower()
-                url_eq_idx = content_str.find("url=")
+                content_str = meta_tag["content"]
+                content_str_length = len(meta_tag["content"])
+                url_eq_idx = content_str.lower().find("url=")
+
                 if url_eq_idx >= 0:
-                    url = meta_tag["content"][url_eq_idx + 4:]
+                    if content_str[url_eq_idx + 4] in ("\"", "'"):
+                        url_eq_idx += 1
+                        if content_str.endswith(("\"", "'")):
+                            content_str_length -= 1
+                    url = content_str[url_eq_idx + 4:content_str_length]
                     if url:
                         urls.add(self.make_absolute(url))
         return [url for url in urls if url]
@@ -756,7 +761,7 @@ class Page:
                 "tel": "0606060606",
                 "text": "default",
                 "time": "13:37",
-                "url": "https://wapiti.sourceforge.io/",
+                "url": "https://wapiti-scanner.github.io/",
                 "username": "alice",
                 "week": "2019-W24"
             }
@@ -814,9 +819,9 @@ class Page:
                 form_actions.add(self.make_absolute(input_field["formaction"].strip() or self.url))
 
             for button_field in form.find_all("button"):
-                if "name" in button_field.attrs and "value" in button_field.attrs:
+                if "name" in button_field.attrs:
                     input_name = button_field["name"]
-                    input_value = button_field["value"]
+                    input_value = button_field.get("value", "")
                     if method == "GET":
                         get_params.append([input_name, input_value])
                     else:
