@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# This file is part of the Wapiti project (https://wapiti.sourceforge.io)
-# Copyright (C) 2008-2021 Nicolas Surribas
+# This file is part of the Wapiti project (https://wapiti-scanner.github.io)
+# Copyright (C) 2008-2022 Nicolas Surribas
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -249,7 +249,10 @@ class Request:
         self._link_depth = link_depth
         parsed = urlparse(self._resource_path)
         self._file_path = parsed.path
-        self._hostname = parsed.netloc
+        self._hostname = parsed.hostname
+        self._scheme = parsed.scheme or ""
+        self._netloc = parsed.netloc
+
         self._port = 80
         if parsed.port is not None:
             self._port = parsed.port
@@ -448,6 +451,14 @@ class Request:
     def hostname(self) -> str:
         return self._hostname
 
+    @property
+    def netloc(self) -> str:
+        return self._netloc
+
+    @property
+    def scheme(self) -> str:
+        return self._scheme.lower()
+
     @property
     def port(self):
         return self._port
